/*
Copyright (c) 1998 Peter Zelezny.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

// LASTLOG by Erik Scrafford <eriks@chilisoft.com>.

#include "style.h"
#include "xchat.h"

extern GdkFont *dialog_font_normal;
extern GdkColor colors[];
extern struct xchatprefs prefs;

extern char *nocasestrstr(char *text, char *tofind);


GtkWidget *lastlog_window, *lastlog_text;
struct session *lastlog_last_sess = 0;


void search_lastlog(struct session *_sess, char *search)
{
   char line[512];
   char *start, *end;
   struct session *sess;
   char title[270];

   if (_sess == NULL) { sess = lastlog_last_sess; } else { sess = _sess; lastlog_last_sess = _sess; }
   gtk_text_backward_delete(GTK_TEXT(lastlog_text), gtk_text_get_length(GTK_TEXT(lastlog_text)));
   if (search == NULL) return;
   
   start = GTK_TEXT(sess->textgad)->text;
   gtk_text_freeze(GTK_TEXT(lastlog_text));
   while ((end = strchr(start, '\n')) && ((end - start) <= GTK_TEXT(sess->textgad)->gap_position))
   {
      strncpy(line, start, end-start);
      line[end-start] = 0;
      if (nocasestrstr(line, search))   
      {
	 strcat(line, "\n");
	 //	 PrintTextRaw(lastlog_text, line, prefs.bg_color, prefs.fg_color, font_normal, font_bold);
	 gtk_text_insert(GTK_TEXT(lastlog_text), dialog_font_normal, &colors[prefs.dialog_fg_color], &colors[prefs.dialog_bg_color], line, strlen(line));
	 //      g_print("%s", line);
      }
      start=++end;
   }
   gtk_text_thaw(GTK_TEXT(lastlog_text));
   
   {
     GtkAdjustment *adj = (GTK_TEXT(lastlog_text))->vadj;
     gtk_adjustment_set_value(adj, adj->upper - adj->lower - adj->page_size);
   }

   sprintf(title, "Lastlog - %s", search);
   gtk_window_set_title(GTK_WINDOW(lastlog_window), title);
}

void hide_lastlog(GtkWidget *wid, gpointer *data)
{
   gtk_widget_destroy(lastlog_window);
   lastlog_window = NULL;
}

void lastlog_entry(GtkWidget *wid, gpointer *data)
{
   search_lastlog(NULL, GTK_ENTRY(wid)->text);
   gtk_editable_delete_text(GTK_EDITABLE(wid), 0, 255);
   //   gtk_editable_delete_selection(GTK_EDITABLE(wid));
}

void show_lastlog_window(void)
{
   GtkWidget *hbox, *vbox, *wid, *lastlog_input;

   if (lastlog_window == NULL)
   {
      lastlog_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
      gtk_widget_set_usize(lastlog_window, 400, 500);
      gtk_window_set_title(GTK_WINDOW(lastlog_window), "Lastlog");
      gtk_signal_connect(GTK_OBJECT(lastlog_window), "destroy", GTK_SIGNAL_FUNC(hide_lastlog), lastlog_window);
      
      vbox = gtk_vbox_new(FALSE, 0);
      gtk_container_border_width(GTK_CONTAINER(vbox), 3);
      gtk_container_add(GTK_CONTAINER(lastlog_window), vbox);
      gtk_widget_show(vbox);
      
      hbox = gtk_hbox_new(FALSE, 0);
      gtk_container_add(GTK_CONTAINER(vbox), hbox);
      gtk_widget_show(hbox);
      
      lastlog_text = gtk_text_new(0, 0);
      {
	 GtkStyle *style = gtk_style_new();
	 memcpy(style, gtk_widget_get_style(lastlog_text), sizeof(GtkStyle));
	 memcpy(style->base, &colors[prefs.dialog_bg_color], sizeof(GdkColor));
	 memcpy(style->bg, &colors[prefs.dialog_bg_color], sizeof(GdkColor));
	 memcpy(style->fg, &colors[prefs.dialog_fg_color], sizeof(GdkColor));
	 gtk_widget_set_style(lastlog_text, style);
      }
      gtk_text_set_word_wrap(GTK_TEXT(lastlog_text), TRUE);
      gtk_text_set_editable(GTK_TEXT(lastlog_text), FALSE);
      gtk_container_add(GTK_CONTAINER(hbox), lastlog_text);
      gtk_widget_show(lastlog_text);
      
      wid = gtk_vscrollbar_new(GTK_TEXT(lastlog_text)->vadj);
      gtk_box_pack_start(GTK_BOX(hbox), wid, FALSE, FALSE, 0);
      gtk_widget_show(wid);
      
      lastlog_input = gtk_entry_new_with_max_length( 255 );
      gtk_box_pack_end(GTK_BOX(vbox), lastlog_input, FALSE, FALSE, 0);
      gtk_signal_connect(GTK_OBJECT(lastlog_input), "activate", GTK_SIGNAL_FUNC(lastlog_entry), NULL);
      gtk_widget_show(lastlog_input);
      gtk_widget_grab_focus(lastlog_input);

      gtk_widget_show(lastlog_window);
   }
}
